"""secrets plural + type

Revision ID: ee4924ac4f60
Revises: 9de11ef8ba39
Create Date: 2026-01-21 15:02:43.409666

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision: str = 'ee4924ac4f60'
down_revision: Union[str, Sequence[str], None] = '9de11ef8ba39'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('px_cloud_secrets',
    sa.Column('target_pve', sa.String(length=253), nullable=False),
    sa.Column('secret_name', sa.String(length=253), nullable=False),
    sa.Column('secret_data', postgresql.JSONB(astext_type=sa.Text()), nullable=False),
    sa.Column('secret_type', sa.String(length=50), nullable=True),
    sa.PrimaryKeyConstraint('target_pve', 'secret_name')
    )
    op.drop_table('px_cloud_secret')
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('px_cloud_secret',
    sa.Column('target_pve', sa.VARCHAR(length=253), autoincrement=False, nullable=False),
    sa.Column('secret_name', sa.VARCHAR(length=253), autoincrement=False, nullable=False),
    sa.Column('secret_data', postgresql.JSONB(astext_type=sa.Text()), autoincrement=False, nullable=False),
    sa.PrimaryKeyConstraint('target_pve', 'secret_name', name=op.f('px_cloud_secret_pkey'))
    )
    op.drop_table('px_cloud_secrets')
    # ### end Alembic commands ###



"""ext control plane

Revision ID: d9b711555be8
Revises: 7868bcd05006
Create Date: 2025-09-10 09:05:05.736095

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = 'd9b711555be8'
down_revision: Union[str, Sequence[str], None] = '7868bcd05006'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('k8s_ext_control_plan',
    sa.Column('stack_fqdn', sa.String(length=253), nullable=False),
    sa.Column('zone', sa.String(length=253), nullable=True),
    sa.Column('name', sa.String(length=253), nullable=True),
    sa.PrimaryKeyConstraint('stack_fqdn')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('k8s_ext_control_plan')
    # ### end Alembic commands ###



from sqlalchemy import Column, Integer, String, Boolean, SmallInteger, Text
from sqlalchemy.dialects.postgresql import MACADDR, INET, JSONB, insert, ENUM
from sqlalchemy import create_engine, MetaData, Table, select, delete, update
from sqlalchemy.orm import declarative_base
from alembic.config import Config
from alembic import command
import os

Base = declarative_base()

class BindDomains(Base):
  __tablename__ = "bind_domains"

  domain = Column(String(253), primary_key=True)
  stack_fqdn = Column(String(253), primary_key=True)


class AcmeX509(Base):
  __tablename__ = "acme_x509"

  stack_fqdn = Column(String(253), primary_key=True)
  config = Column(JSONB)
  ec_csr = Column(JSONB)
  ec_crt = Column(JSONB)
  k8s = Column(JSONB)


class KeaReservations(Base):
  __tablename__ = "kea_reservations"

  mac = Column(MACADDR, primary_key=True)
  ip = Column(INET) # nullable
  hostname = Column(String(253), nullable=False)
  client_classes = Column(String(1000)) # csv seperated client classes
  stack_fqdn = Column(String(253), nullable=False)
  machine_type = Column(String(50), nullable=False)


class KeaClientClassDefs(Base):
  __tablename__ = "kea_client_class_defs"

  stack_fqdn = Column(String(253), nullable=False)
  class_name = Column(String(253), primary_key=True)
  class_content = Column(JSONB, nullable=False)


# todo: rename just ingress rules
class K8SIngressRules(Base):
  __tablename__ = "k8s_ingress_rules"

  zone = Column(String(253), primary_key=True)
  name = Column(String(253), primary_key=True)
  stack_fqdn = Column(String(253), primary_key=True)
  proxy_stack_fqdn = Column(String(253), nullable=False)
  external = Column(Boolean, default=False)
  rule_len = Column(Integer, nullable=False)
  is_k8s = Column(Boolean, nullable=False)


# todo: rename to just tcp proxies
class K8STcpProxies(Base):
  __tablename__ = "k8s_tcp_proxies"

  proxy_name = Column(String(253), nullable=False)
  haproxy_port = Column(SmallInteger, primary_key=True)
  node_port = Column(SmallInteger, nullable=False)
  stack_fqdn = Column(String(253), nullable=False)
  # determines backend routing in haproxy, if false will go 
  is_k8s = Column(Boolean, nullable=False) 
  proxy_snippet = Column(Text)
  proxy_stack_fqdn = Column(String(253), primary_key=True)
  external = Column(Boolean, default=False)


class K8SExternalControlPlanes(Base):
  __tablename__ = "k8s_ext_control_planes"

  stack_fqdn = Column(String(253), primary_key=True)
  extra_sans = Column(String(2530), nullable=False) # csvs
  proxy_stack_fqdn = Column(String(253), nullable=False)


# vars are overwrite on exist and have no guarantee that they get cleaned up
# / that the vms still exist if a user just deletes via proxmox ui
# todo: built in some sort of cleanup mechanism
class VirtualMachineVars(Base):
  __tablename__ = "vm_vars_blake"

  cloud_domain = Column(String(253), primary_key=True)
  blake_id = Column(String(50), primary_key=True)
  vm_vars = Column(JSONB, nullable=False)


# is also used for configuration and dynamic discovery
class ProxmoxCloudSecrets(Base):
  __tablename__ = "px_cloud_secrets"

  cloud_domain = Column(String(253), primary_key=True)
  secret_name = Column(String(253), primary_key=True)
  secret_data = Column(JSONB, nullable=False)
  secret_type = Column(String(50), nullable=True)


# apply the migrations to the database
def migrate(conn_str):
  alembic_cfg = Config(os.path.join(os.path.dirname(__file__), 'alembic.ini'))

  alembic_cfg.set_main_option("sqlalchemy.url", conn_str)
  alembic_cfg.set_main_option("prepend_sys_path", os.path.dirname(__file__))

  command.upgrade(alembic_cfg, "head")


# generic read with simple where equal functionality
def alch_read(conn_str, table_name, where_equal_args):
  engine = create_engine(conn_str)
  metadata = MetaData()

  table = Table(table_name, metadata, autoload_with=engine)

  statement = select(table)
  for col_name, val in where_equal_args.items():
    statement = statement.where(table.c[col_name] == val)

  with engine.connect() as conn:
    result = conn.execute(statement)
    
  return [dict(row._mapping) for row in result]


def alch_write(conn_str, table_name, rows):
  engine = create_engine(conn_str)
  metadata = MetaData()

  table = Table(table_name, metadata, autoload_with=engine)

  stmt = insert(table)

  with engine.begin() as conn:
    conn.execute(stmt, rows)


def alch_update(conn_str, table_name, values, whereclause):
  engine = create_engine(conn_str)
  metadata = MetaData()
  table = Table(table_name, metadata, autoload_with=engine)

  stmt = update(table).values(**values)

  for col, val in whereclause.items():
    stmt = stmt.where(table.c[col] == val)

  with engine.begin() as conn:
    conn.execute(stmt)


def alch_upsert(conn_str, table_name, values, conflict_columns):
  engine = create_engine(conn_str)
  metadata = MetaData()
  table = Table(table_name, metadata, autoload_with=engine)

  stmt = insert(table).values(**values)

  # update every column except conflict columns that were passed in the values
  # this is the same as doing SET column_x = EXCLUDED.column_x
  update_dict = {c: getattr(stmt.excluded, c) for c in values if c not in conflict_columns}

  stmt = stmt.on_conflict_do_update(
    index_elements=[table.c[c] for c in conflict_columns],
    set_=update_dict
  )

  with engine.begin() as conn:
    conn.execute(stmt)


def alch_delete(conn_str, table_name, where_equal_args):
  engine = create_engine(conn_str)
  metadata = MetaData()

  table = Table(table_name, metadata, autoload_with=engine)

  stmt = delete(table)
  for col_name, val in where_equal_args.items():
    stmt = stmt.where(table.c[col_name] == val)

  with engine.begin() as conn: 
    result = conn.execute(stmt)
    return result.rowcount
  


"""slop firewall seperation

Revision ID: fdcb5aa33b76
Revises: 04398db10434
Create Date: 2025-11-21 17:33:31.778613

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision: str = 'fdcb5aa33b76'
down_revision: Union[str, Sequence[str], None] = '04398db10434'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('kea_client_class_defs',
    sa.Column('stack_fqdn', sa.String(length=253), nullable=False),
    sa.Column('class_name', sa.String(length=253), nullable=False),
    sa.Column('class_content', postgresql.JSONB(astext_type=sa.Text()), nullable=False),
    sa.PrimaryKeyConstraint('class_name')
    )

    # manual pkey change
    op.drop_constraint('kea_reservations_pkey', 'kea_reservations', type_='primary')
    op.create_primary_key('kea_reservations_pkey', 'kea_reservations', ['mac'])

    op.add_column('kea_reservations', sa.Column('client_classes', sa.String(length=1000), nullable=True))
    op.alter_column('kea_reservations', 'ip',
               existing_type=postgresql.INET(),
               nullable=True)
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # manual change
    op.drop_constraint('kea_reservations_pkey', 'kea_reservations', type_='primary')
    op.create_primary_key('kea_reservations_pkey', 'kea_reservations', ['ip'])

    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('kea_reservations', 'ip',
               existing_type=postgresql.INET(),
               nullable=False)
    op.drop_column('kea_reservations', 'client_classes')
    op.drop_table('kea_client_class_defs')
    # ### end Alembic commands ###



"""len rules + enforcements

Revision ID: 24a548bfce3e
Revises: 944a8fd5d5bc
Create Date: 2025-09-10 17:12:21.594162

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '24a548bfce3e'
down_revision: Union[str, Sequence[str], None] = '944a8fd5d5bc'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('k8s_ext_control_planes', 'zone',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_ext_control_planes', 'name',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_ext_control_planes', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)

    # custom migration logic
    op.add_column('k8s_ingress_rules', sa.Column('rule_len', sa.Integer(), nullable=True))

    conn = op.get_bind()
    conn.execute(sa.text("""
        UPDATE k8s_ingress_rules
        SET rule_len = length(zone) + length(name)
    """))

    op.alter_column('k8s_ingress_rules', 'rule_len', nullable=False)

    op.alter_column('k8s_ingress_rules', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_masters', 'hostname',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_masters', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_masters', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_tcp_proxies', 'proxy_name',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_tcp_proxies', 'node_port',
               existing_type=sa.SMALLINT(),
               nullable=False)
    op.alter_column('k8s_tcp_proxies', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_workers', 'hostname',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_workers', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('k8s_workers', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('kea_reservations', 'hostname',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    op.alter_column('kea_reservations', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('kea_reservations', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('kea_reservations', 'hostname',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_workers', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_workers', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_workers', 'hostname',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_tcp_proxies', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_tcp_proxies', 'node_port',
               existing_type=sa.SMALLINT(),
               nullable=True)
    op.alter_column('k8s_tcp_proxies', 'proxy_name',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_masters', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_masters', 'stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_masters', 'hostname',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_ingress_rules', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.drop_column('k8s_ingress_rules', 'rule_len')
    op.alter_column('k8s_ext_control_planes', 'proxy_stack_fqdn',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_ext_control_planes', 'name',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    op.alter_column('k8s_ext_control_planes', 'zone',
               existing_type=sa.VARCHAR(length=253),
               nullable=True)
    # ### end Alembic commands ###



"""fix

Revision ID: 3c95509a5de9
Revises: d9b711555be8
Create Date: 2025-09-10 16:03:18.769927

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '3c95509a5de9'
down_revision: Union[str, Sequence[str], None] = 'd9b711555be8'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('k8s_ext_control_plane',
    sa.Column('stack_fqdn', sa.String(length=253), nullable=False),
    sa.Column('zone', sa.String(length=253), nullable=True),
    sa.Column('name', sa.String(length=253), nullable=True),
    sa.PrimaryKeyConstraint('stack_fqdn')
    )
    op.drop_table('k8s_ext_control_plan')
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('k8s_ext_control_plan',
    sa.Column('stack_fqdn', sa.VARCHAR(length=253), autoincrement=False, nullable=False),
    sa.Column('zone', sa.VARCHAR(length=253), autoincrement=False, nullable=True),
    sa.Column('name', sa.VARCHAR(length=253), autoincrement=False, nullable=True),
    sa.PrimaryKeyConstraint('stack_fqdn', name=op.f('k8s_ext_control_plan_pkey'))
    )
    op.drop_table('k8s_ext_control_plane')
    # ### end Alembic commands ###
